/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.connectionpool;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * Context of Connection Pool
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-08
 */
public class ConnectionPoolContext {

	private static Context ctx;
	
	private final static ConnectionPoolContext cpc;
	
	static {
		cpc = new ConnectionPoolContext();
		
		//initialize context,it's for preparing datasource
		Hashtable<String,Object> env = new Hashtable<String,Object>();
		env.put(Context.INITIAL_CONTEXT_FACTORY, 
				"com.nonesole.persistence.jndi.PersistenceInitCtxFactory");
		try {
			ctx = new InitialContext(env);
		} catch (NamingException e) {
			e.printStackTrace();
		}		
	}
	
	private ConnectionPoolContext(){}
	
	/**
	 * Return ConnectionPool Context Object.
	 * @return ConnectionPoolContext
	 * */
	public synchronized static ConnectionPoolContext getInstance(){
		return cpc;
	}
	
	/**
	 * Return the only one context.
	 * @return Context
	 * */
	public synchronized Context getInitialContext(){
		return ctx;
	}
	
	/**
	 * Return datasource
	 * @param name datasource's name
	 * @return datasource
	 * @throws NamingException 
	 * */
	public synchronized DataSource getDataSource(String name) throws NamingException{
		if ( null == name || "".equals(name) )
			return null;
		return (DataSource)ctx.lookup(name);
	}
	
}
