/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.connectionpool;

import java.io.PrintWriter;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.sql.ConnectionPoolDataSource;
import javax.sql.PooledConnection;

import com.nonesole.persistence.connectionpool.parser.DriverProperties;
import com.nonesole.persistence.exception.OperationsException;

/**
 * Datasource for connection pool
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-04
 */
public class DriverConnectionPoolDataSource implements ConnectionPoolDataSource{

	//datasource config object
	private DriverProperties driver;
	
	//exception head
	private final static String REPORTER = "DriverConnectionPoolDataSource";
	private final static String KEY_ENTER = "\r\n";
	
	private DriverConnectionPoolDataSource(DriverProperties dr) 
	throws OperationsException{
		driver = dr;
		if( !driver.isValid() )
			throw new OperationsException("DataSource Config is failure.");
		try {
			Class.forName(driver.getDriverClass()).newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * ConnectionPool's datasource new object
	 * @param dr DriverConnectionPoolDataSource
	 * @throws OperationsException 
	 * */
	protected static DriverConnectionPoolDataSource getInstance(
			DriverProperties dr) throws OperationsException{
		return new DriverConnectionPoolDataSource(dr);
	}
	
	//it doesnt't work
	private PrintWriter writer;
	
	public PrintWriter getLogWriter() throws SQLException {
		return writer;
	}


	public void setLogWriter(PrintWriter out) throws SQLException {
		this.writer = out;
	}

	public synchronized PooledConnection getPooledConnection() throws SQLException {
		return getPooledConnection(null,null);
	}

	public synchronized PooledConnection getPooledConnection(String user, String password)
			throws SQLException {
		if ( null != user ) {
			driver.setUrl(user);
			driver.setPwd(password);
		} 
		try {			
			//create connection to database
			return new DriverPooledConnection(
							DriverManager.getConnection(
								driver.getUrl(),
								driver.getUser(),
								driver.getPwd()));
		} catch (SQLException e) {
			new OperationsException(REPORTER + e.getMessage() + KEY_ENTER,e.getCause());
		}
		return null;
	}

	//max login time
	private int maxLoginTime = 0; 
	
	public void setLoginTimeout(int seconds) throws SQLException {
		this.maxLoginTime = seconds;
	}

	public int getLoginTimeout() throws SQLException {
		return this.maxLoginTime;
	}
	
}
