/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.connectionpool.parser;

/**
 * Properties of Connection Pool
 * @author JACK
 * @version 1.0 - build in 2009-07-01
 */
public class ConnectionPoolProperties {

	private String poolConfigName;
	private int maxStatements;
	private int initialPoolSize;
	private int minPoolSize;
	private int maxPoolSize;
	private int maxidleTime;
	private int propertyCycle;
		
	/**
	 * Current object is whether or not valid. 
	 * @return true - valid
	 * */
	public boolean isValid(){
		if ( initialPoolSize < minPoolSize 
				|| initialPoolSize > maxPoolSize
				|| minPoolSize > maxPoolSize
				|| initialPoolSize < 0
				|| minPoolSize < 0
				|| maxPoolSize < 0
				|| maxidleTime < 0
				|| propertyCycle < 0
				|| maxStatements < 0
				|| null == poolConfigName
				|| "".equals(poolConfigName) ) 
			return false;
		return true;
	}
	
	/**
	 * Rewrite toString method 
	 * @return String
	 * */
	public String toString(){
		return this.poolConfigName;
	}
	
	/**
	 * Rewrite equals method 
	 * this place do not need rewrite hashCode method.
	 * @return true or false
	 * */
	public boolean equals(Object obj){
		if ( obj instanceof ConnectionPoolProperties ) {
			ConnectionPoolProperties cpp = (ConnectionPoolProperties)obj;
			if ( this.poolConfigName.equals(cpp.getPoolConfigName()) 
					&& this.initialPoolSize == cpp.getInitialPoolSize()
					&& this.minPoolSize == cpp.getMinPoolSize()
					&& this.maxPoolSize == cpp.getMaxPoolSize()
					&& this.maxidleTime == cpp.getMaxidleTime()
					&& this.propertyCycle == cpp.getPropertyCycle()
					&& this.maxStatements == cpp.getMaxStatements() )
				return true;
		}
		return false;
	}
		
	/**
	 * @return the poolConfigName
	 */
	public String getPoolConfigName() {
		return poolConfigName;
	}

	/**
	 * @param poolConfigName the poolConfigName to set
	 */
	public void setPoolConfigName(String poolConfigName) {
		this.poolConfigName = poolConfigName;
	}
	/**
	 * @return the maxStatements
	 */
	public int getMaxStatements() {
		return maxStatements;
	}
	/**
	 * @param maxStatements the maxStatements to set
	 */
	public void setMaxStatements(int maxStatements) {
		this.maxStatements = maxStatements;
	}
	/**
	 * @return the initialPoolSize
	 */
	public int getInitialPoolSize() {
		return initialPoolSize;
	}
	/**
	 * @param initialPoolSize the initialPoolSize to set
	 */
	public void setInitialPoolSize(int initialPoolSize) {
		this.initialPoolSize = initialPoolSize;
	}
	/**
	 * @return the minPoolSize
	 */
	public int getMinPoolSize() {
		return minPoolSize;
	}
	/**
	 * @param minPoolSize the minPoolSize to set
	 */
	public void setMinPoolSize(int minPoolSize) {
		this.minPoolSize = minPoolSize;
	}
	/**
	 * @return the maxPoolSize
	 */
	public int getMaxPoolSize() {
		return maxPoolSize;
	}
	/**
	 * @param maxPoolSize the maxPoolSize to set
	 */
	public void setMaxPoolSize(int maxPoolSize) {
		this.maxPoolSize = maxPoolSize;
	}
	/**
	 * @return the maxidleTime
	 */
	public int getMaxidleTime() {
		return maxidleTime;
	}
	/**
	 * @param maxidleTime the maxidleTime to set
	 */
	public void setMaxidleTime(int maxidleTime) {
		this.maxidleTime = maxidleTime;
	}
	/**
	 * @return the propertyCycle
	 */
	public int getPropertyCycle() {
		return propertyCycle;
	}
	/**
	 * @param propertyCycle the propertyCycle to set
	 */
	public void setPropertyCycle(int propertyCycle) {
		this.propertyCycle = propertyCycle;
	}
}
