/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.connectionpool.parser;

/**
 * JDBC driver
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-01
 */
public class DriverProperties {

	private String dataSourceName;
	private String driverClass;
	private String url;
	private String user;
	private String pwd;
	private boolean associateConnectionPool = false;//JDBC or ConnectionPool
	private String connectionPoolName;//if associateConnectionPool = false ,do not need this property
	
	/**
	 * Current object is whether or not valid. 
	 * @return true - valid
	 * */
	public boolean isValid(){
		if ( null == this.dataSourceName 
				|| null == this.driverClass
				|| null == this.url
				|| null == this.user
				|| null == this.pwd
				|| "".equals(this.dataSourceName)
				|| "".equals(this.driverClass)
				|| "".equals(this.url)
				|| "".equals(this.user) ) 
			return false;
		if ( associateConnectionPool && 
				( null == this.connectionPoolName
				|| "".equals(this.connectionPoolName)) )
			return false;
		return true;
	}
	
	/**
	 * Rewrite toString method 
	 * @return String
	 * */
	public String toString(){
		return this.dataSourceName;
	}
	
	/**
	 * Rewrite equals method 
	 * this place do not need rewrite hashCode method.
	 * @return true or false
	 * */
	public boolean equals(Object obj){
		if ( obj instanceof DriverProperties ) {
			DriverProperties dp = (DriverProperties)obj;
			if ( this.dataSourceName.equals(dp.getDataSourceName()) 
					&& this.driverClass.equals(dp.getDriverClass())
					&& this.url.equals(dp.getUrl())
					&& this.user.equals(dp.getUser())
					&& this.pwd.equals(dp.getPwd()) 
					&& this.associateConnectionPool == dp.isAssociateConnectionPool()
					&& ( this.associateConnectionPool == true 
							&& this.connectionPoolName.equals(dp.getConnectionPoolName()) ) )
				return true;
		}
		return false;
	}
	
	
	/**
	 * @return the associateConnectionPool
	 */
	public boolean isAssociateConnectionPool() {
		return associateConnectionPool;
	}
	/**
	 * @param associateConnectionPool the associateConnectionPool to set
	 */
	public void setAssociateConnectionPool(boolean associateConnectionPool) {
		this.associateConnectionPool = associateConnectionPool;
	}
	/**
	 * @return the connectionPoolName
	 */
	public String getConnectionPoolName() {
		return connectionPoolName;
	}
	/**
	 * @param connectionPoolName the connectionPoolName to set
	 */
	public void setConnectionPoolName(String connectionPoolName) {
		this.connectionPoolName = connectionPoolName;
	}
	/**
	 * @return the driverClass
	 */
	public String getDriverClass() {
		return driverClass;
	}
	/**
	 * @param driverClass the driverClass to set
	 */
	public void setDriverClass(String driverClass) {
		this.driverClass = driverClass;
	}
	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}
	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}
	/**
	 * @return the user
	 */
	public String getUser() {
		return user;
	}
	/**
	 * @param user the user to set
	 */
	public void setUser(String user) {
		this.user = user;
	}
	/**
	 * @return the pwd
	 */
	public String getPwd() {
		return pwd;
	}
	/**
	 * @param pwd the pwd to set
	 */
	public void setPwd(String pwd) {
		this.pwd = pwd;
	}
	/**
	 * @return the dataSourceName
	 */
	public String getDataSourceName() {
		return dataSourceName;
	}
	/**
	 * @param dataSourceName the dataSourceName to set
	 */
	public void setDataSourceName(String dataSourceName) {
		this.dataSourceName = dataSourceName;
	}
}
