/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.exception;

import java.io.PrintWriter;

import com.nonesole.persistence.main.FrameConfigConstruction;
import com.nonesole.persistence.main.FrameEnvService;
import com.nonesole.persistence.tools.DateToolkit;
import com.nonesole.persistence.tools.FileToolkit;

/**
 * Exception of persistence<br>
 * Save exception in logs.
 * @author JACK Lee
 * @version 1.0 - build in 2009-06-28
 */
public class OperationsException extends Exception{
	
	private static final long serialVersionUID = 8511972741058192330L;
	private static final String FILE_PATH;
	private static final String TIPS;
	
	static {
		FILE_PATH = FrameEnvService
						.getInstance()
						.getProperty(FrameConfigConstruction.DEFAULT_LOG);
		TIPS = "Persistence exception";
	}
	
	public OperationsException(){ this(""); }
	
	public OperationsException(String message){ this(message,null); }
	
	public OperationsException(Throwable cause){ this("",cause); }
	
	public OperationsException(String message, Throwable cause){ 
		this(message,cause,null);
	}
		
	/**
	 * Throw exception and save it in log file
	 * @param message - String
	 * @param cause - Throwable
	 * @param logFilePath String
	 * */
	public OperationsException(String message, 
				Throwable cause,String logFilePath){ 
		super(new StringBuilder()
				.append(TIPS).append(" : ")
				.append((null == message?"":message)).toString(),cause);
		if ( null == logFilePath )
			logFilePath = FILE_PATH;
		
		PrintWriter pw = FileToolkit.getPrintWriter(logFilePath);
		pw.write(new StringBuilder()
					.append(FileToolkit.LINE_SEPARATOR)
					.append("[ Occurred at this time : ")
					.append(DateToolkit.now())
					.append(" ]")
					.append(FileToolkit.LINE_SEPARATOR)
					.append(FileToolkit.LINE_SEPARATOR).toString());
		this.printStackTrace(pw);
		pw.close();
		pw = null;
			
		
		
	}

}
