/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.properties.PropertiesOperation;
import com.nonesole.persistence.file.se.SerializationOperation;
import com.nonesole.persistence.file.txt.TXTOperation;
import com.nonesole.persistence.file.xml.XMLOperation;

/**
 * Get beans which implements interface IFileOperations
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-20
 */
public class FileOperationsFactory {

	public final static int TXT_OPERATION = 1;
	public final static int XML_OPERATION = 2;
	public final static int PROPERTY_OPERATION = 3;
	public final static int SERIALIZATION_OPERATION = 4;
	private static FileOperationsFactory FOF;
	static {
		FOF = new FileOperationsFactory();
	}
	private FileOperationsFactory(){}
	
	/**
	 * Return FileOperationsFactory Object
	 * @return FileOperationsFactory
	 * */
	public static FileOperationsFactory getInstance(){
		return FOF;
	}
	
	/**
	 * Get operation by parameter type
	 * @param operation type
	 * @return Operation implements IFileOperations
	 * */
	public IFileOperations getOperation(int type) throws OperationsException{
		switch(type){
		case TXT_OPERATION:
			return TXTOperation.getInstance();
		case XML_OPERATION:
			return XMLOperation.getInstance();
		case PROPERTY_OPERATION:
			return PropertiesOperation.getInstance();
		case SERIALIZATION_OPERATION:
			return SerializationOperation.getInstance();
		default :
			throw new OperationsException(
					"Error file type was chosen.");
		}
	}			
}
