/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file;

import java.net.URL;
import java.util.List;

import com.nonesole.persistence.exception.OperationsException;

/**
 * File Operation Interface<br>
 * <pre>
 * Notice:
 *   1.All the objects as parameter must have isValid method.
 *     They must check themselves and tell frame whether or
 *     not they are valid.
 *   2.The primary keys of the parameter objects come from 
 *     toString() method.
 *   3.Parameter object creator must decide how to rewrite
 *     equals() and hashCode() method.
 *   4.The parameter url is where the file will be saved.  
 * </pre>
 * @author JACK LEE 
 * @version 1.0 - build in 2009-07-20
 */
public interface IFileOperations {

	/**
	 * Insert one object into file system
	 * @param obj Object
	 * @param url - file URL
	 * @return Object - obj after dealing with
	 * */
	public Object insert(Object obj,URL url) throws OperationsException;
	
	/**
	 * Update one object of file system
	 * @param obj Object
	 * @param url - file URL
	 * @return Object - obj after dealing with
	 * */
	public Object update(Object obj,URL url) throws OperationsException;
	
	/**
	 * Delete one object of file system
	 * @param obj Object
	 * @param url - file URL
	 * */
	public void delete(Object obj,URL url) throws OperationsException;
	
	/**
	 * Query.<br>
	 * Those classed who implement this interface will set search condition
	 * by themselves.So it uses Object(common condition object) here. 
	 * @param obj Object
	 * @param url - file URL
	 * @return result of search
	 * */
	public List<Object> query(Object obj,URL url) throws OperationsException;		

	/**
	 * Insert objects into file system
	 * @param obj list of Object
	 * @param url - file URL
	 * @return Object - list of objs after dealing with
	 * */
	public List<Object> batchInsert(List<Object> obj,URL url) throws OperationsException;
	
	/**
	 * Update objects of file system
	 * @param obj list of Object
	 * @param url - file URL
	 * @return Object - list of objs after dealing with
	 * */
	public List<Object> batchUpdate(List<Object> obj,URL url) throws OperationsException;
	
	/**
	 * Delete objects of file system
	 * @param obj list of Object
	 * @param url - file URL
	 * */
	public void batchDelete(List<Object> obj,URL url) throws OperationsException;
	
}
