/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.properties.PropertiesTransformer;
import com.nonesole.persistence.file.se.SerializationTransformer;
import com.nonesole.persistence.file.txt.TXTTransformer;
import com.nonesole.persistence.file.xml.XMLTransformer;

/**
 * Transformer Factory
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-26
 */
public class TransformerFactory {

	public final static int PROPERTIES_FILE = 1;
	public final static int TXT_FILE = 2;	
	public final static int XML_FILE = 3;
	public final static int SERIALIZATION_FILE = 4;	
	
	public final static String PROPERTIES_FILE_END = ".properties";
	public final static String TXT_FILE_END = ".txt";	
	public final static String XML_FILE_END = ".xml";
	public final static String SERIALIZATION_FILE_END = ".ser";	
	
	private final static TransformerFactory TF = new TransformerFactory();
	private TransformerFactory(){}
	/**
	 * Get object of factory.
	 * @return TransformerFactory
	 * */
	public static TransformerFactory getInstance(){
		return TF;
	}
	
	/**
	 * Get some file transformer by file type(Integer format).
	 * @param file type
	 * @return object of IFileTransformer
	 * */
	public IFileTransformer getFileTransformer(int filetype) throws OperationsException{
		switch(filetype){
		case PROPERTIES_FILE:
			return PropertiesTransformer.getInstance();
		case TXT_FILE:
			return TXTTransformer.getInstance();
		case XML_FILE:
			return XMLTransformer.getInstance();
		case SERIALIZATION_FILE:
			return SerializationTransformer.getInstance();
		default :
			throw new OperationsException(
					"Error file type was chosen.");
		}
	}
	
	/**
	 * Get some file transformer by file type(String format).
	 * @param file type(String)
	 * @return object of IFileTransformer
	 * */
	public IFileTransformer getFileTransformer(String filePath) throws OperationsException {
		
		if ( filePath.endsWith(TransformerFactory.PROPERTIES_FILE_END) ) {
			return PropertiesTransformer.getInstance();
		} else if ( filePath.endsWith(TransformerFactory.SERIALIZATION_FILE_END) ) {
			return SerializationTransformer.getInstance();
		} else if ( filePath.endsWith(TransformerFactory.TXT_FILE_END) ) {
			return TXTTransformer.getInstance();
		} else if ( filePath.endsWith(TransformerFactory.XML_FILE_END) ) {
			return XMLTransformer.getInstance();
		} else {
			throw new OperationsException(
				"Error file type was chosen.");
		}
	}
		
}
