/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.catalog;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import com.nonesole.persistence.exception.OperationsException;

/**
 * Catalog Factory
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-25
 */
public class CatalogFactory {

	private final static CatalogFactory CF = new CatalogFactory();
	
	/**
	 * Catch of catalog object for rapid working
	 * */
	private static Map<String,ICatalog> catalogMap 
						= new HashMap<String,ICatalog>();
	
	private ICatalog cl = null;
	
	public final static CatalogFactory getInstance(){
		return CF;
	}
	
	/**
	 * Get Catalog of filePath
	 * @param filePath 
	 * @return ICatalog object
	 * */
	public final CatalogFactory reload(String filePath) 
	throws OperationsException, IOException{
		cl = new Catalog(
				new StringBuilder()
					.append(filePath)
					.append("/")
					.append(getDefaultFileName())
						.toString().replace("//", "/"));
		catalogMap.put(filePath, cl);
		return this;
	}
	
	/**
	 * Rapidly get Catalog of filePath
	 * @param filePath 
	 * @return ICatalog object
	 * */
	public final ICatalog getMemoryCatalog(String filePath) 
	throws OperationsException, IOException{
		cl = catalogMap.get(filePath);
		if ( null == cl ){
			reload(filePath);
			cl = catalogMap.get(filePath);
		}
		return cl;
	}
	
	/**
	 * Get default manage file name of catalog service 
	 * from config.properties file
	 * @return fileName
	 * */
	public final String getDefaultFileName() throws IOException{
		Properties prop = new Properties();   
        InputStream inStream = 
        	CatalogFactory.class 
                .getResourceAsStream("config.properties");   
        prop.load(inStream);   
        return prop.getProperty("catalog"); 
	}
	
}
