/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.catalog;

import com.nonesole.persistence.exception.OperationsException;

/**
 * Interface of controlling catalog service
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-25
 * */
public interface ICatalog {

	/**
	 * Bind CatalogEntry
	 * @param ce - CatalogEntry object 
	 * @throws OperationsException 
	 * */
	public abstract void bind(CatalogEntry ce) throws OperationsException;

	/**
	 * Rebind CatalogEntry
	 * @param ce - CatalogEntry object 
	 * @throws OperationsException 
	 * */
	public abstract void rebind(CatalogEntry ce) throws OperationsException;

	/**
	 * Unbind CatalogEntry
	 * @param ce - CatalogEntry object 
	 * @throws OperationsException 
	 * */
	public abstract void unbind(CatalogEntry ce) throws OperationsException;

	/**
	 * Search CatalogEntry by id
	 * @param id 
	 * @return CatalogEntry
	 * */
	public abstract CatalogEntry lookup(String id);

	/**
	 * Get next CatalogEntry object
	 * @return CatalogEntry
	 * */
	public abstract CatalogEntry next();
	
	/**
	 * Whether or not exist next object
	 * @return true - there is one object next
	 * */
	public abstract boolean hasNext();
	
	/**
	 * Reset all of internal variables  
	 * */
	public abstract void rewind();
	
	
	/**
	 * Destroy current object
	 * @throws InterruptedException 
	 * */
	public abstract void destroy() throws InterruptedException;

	/**
	 * Saving data is whether or not synchronization.
	 * @return true - synchronization
	 */
	public abstract boolean isSynchronization();

	/**
	 * Set synchronization.
	 * @param isSynchronization - true: synchronization
	 */
	public abstract ICatalog setSynchronization(boolean isSynchronization);

	/**
	 * Circle time(ms)
	 * @return the circleTime
	 */
	public abstract long getCircleTime();

	/**
	 * Set circle time(ms)
	 * @param circleTime 
	 */
	public abstract ICatalog setCircleTime(long circleTime);
	
	/**
	 * Get folder path
	 * */
	public String getFolderPath();

}