/*
    Persistence v1.05 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.operation.first;

import java.lang.reflect.Field;
import java.net.URI;
import java.util.List;
import java.util.TreeMap;

import com.nonesole.persistence.exception.OperationsException;

/**
 * <pre>
 * In the version 1.04 of persistence,some actions on handling files were too simple to use,
 * like querying files.So it would be changed to deal with files in the new version.
 * More power actions will be provided ,and they can give users more help.    
 * 
 * This interface ,FileAction ,creates all the new actions on handling files.
 * </pre>
 * @author JACK LEE
 * @version 1.04 - build on 2010-10-29
 */
public interface FileAction {
	
	/**
	 * Set file action.
	 * @param child folder
	 * */
	public SeFileAction setChildFolder(String childFolder);
	
	/**
	 * Set root folder of file db
	 * @param url - file URL
	 * @return FileAction 
	 * */
	public FileAction setDBFolder(URI uri) throws OperationsException;
	
	/**
	 * Create new index which made of TreeMap.
	 * @param url - index name
	 * @return TreeMap<?,?> 
	 * */
	public TreeMap<?,?> createIndex(String indexName,Class<?> clazz,Field field) throws OperationsException;
	
	/**
	 * Get new index which made of TreeMap.
	 * @param url - index name
	 * @return TreeMap<?,?> 
	 * */
	public TreeMap<?,?> getIndex(Class<?> clazz,String indexName) throws OperationsException;
	
	/**
	 * Remove index which made of TreeMap.
	 * @param url - index name
	 * @return TreeMap<?,?> 
	 * */
	public boolean removeIndex(Class<?> clazz,String indexName) throws OperationsException;
	
	/**
	 * Insert one object into file system
	 * @param obj Object
	 * @return Object - obj after dealing with
	 * */
	public Object insert(Object obj) throws OperationsException;
	
	/**
	 * Update one object of file system
	 * @param obj Object
	 * @param p Predicate Search condition object
	 * @param replaceObject If users want to change the whole object ,it should be true. 
	 * @return Object - obj after dealing with
	 * */
	public Object update(Object obj,Predicate<?> p,boolean replaceObject) throws OperationsException;
	
	/**
	 * Delete one object of file system
	 * @param obj Object
	 * @param clazz - Class of object which will be deleted
	 * @return the number of the objects which is deleted 
	 * */
	public int delete(Predicate p,Class<?> clazz) throws OperationsException;
	
	/**
	 * Query.<br>
	 * Those classed who implement this interface will set search condition
	 * by themselves.So it uses Object(common condition object) here. 
	 * @param p - search condition
	 * @param clazz - Class of return
	 * @return result of search
	 * */
	public List<?> query(Predicate p,Class<?> clazz) throws OperationsException;		

	/**
	 * Insert objects into file system
	 * @param obj list of Object
	 * @return Object - list of objs after dealing with
	 * */
	public List<?> batchInsert(List<?> obj) throws OperationsException;
	
	/**
	 * Update objects of file system
	 * @param obj Object
	 * @param p Predicate Search condition object
	 * @param replaceObject If users want to change the whole object ,it should be true. 
	 * @return Object - list of objs after dealing with
	 * */
	public List<?> batchUpdate(Object obj,Predicate p,boolean replaceObject) throws OperationsException;
	
}
