/*
    Persistence v1.05 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.operation.first;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.main.FrameEnvService;
import com.nonesole.persistence.tools.FileToolkit;
import com.nonesole.persistence.tools.ReflectToolkit;

/**
 * @author JACK
 * @version 1.0 - build on 2010-11-04
 */
public class SeFileAction  implements FileAction {

	private static final String INDEX_KEY = "index";
	private static final String LIST_KEY = "list";
	private static final String EX_FILE_NAME = ".se";
	private static final String INDEX_EX_FILE_NAME = ".index";
	private static final String INDEX_STORE = "indexs.str";
	private String dbFolder;
	private List<Integer> objFileIndex;
	private String childFolder;
	
	public SeFileAction(){}
		
	/**
	 * Get set which likes db folder path from config file.
	 * @return URL obj
	 * */
	public static URI getConfig(String key){
		try {
			return new URI(FrameEnvService.getInstance().getProperty(key));
		} catch (URISyntaxException e) {
			new OperationsException(
					"It has mistake to get configration form file.",
					e.fillInStackTrace());
			return null;
		}
	}
	
	/**
	 * Get set which likes db folder path from system env.
	 * @return URL obj
	 * */
	public static URL getEnv(String key){
		try {
			return new URL(System.getenv(key));
		} catch (MalformedURLException e) {
			new OperationsException(
					"It has mistake to get configration from system env.",
					e.fillInStackTrace());
			return null;
		}
	}
		
	/**
	 * Get folder of object.
	 * @param object
	 * @return path of folder
	 * */
	private String getObjectFolder(Object obj){
		if ( null == obj ) return null;
		return getObjectFolder(obj.getClass());
	}
	
	/**
	 * Get folder of object.
	 * @param object
	 * @return path of folder
	 * */
	private String getObjectFolder(Class<?> clazz){
		return dbFolder + "/" + clazz.toString()
								     .replaceFirst("class ", "")
								     .replace(".", "/")
						      +(childFolder==null?"":childFolder);
	}
	
	/**
	 * Load index of object from file.
	 * @param objFolder String 
	 * */
	@SuppressWarnings("unchecked")
	private List<Integer> loadObjectFileIndex(String objFolder) 
	throws OperationsException{
		try {
			String s = objFolder+"/index"+EX_FILE_NAME;
			File f = new File(objFolder);
			if ( !f.exists() ) f.mkdirs();
			f = new File(s); 
			if ( !f.exists() ) f.createNewFile();
			List<Integer> l = null;
			if ( f.length() == 0 ) l = new ArrayList<Integer>();
			else l = (List<Integer>) FileToolkit.loadFileToObject(s);
			if ( l.size() == 0 ) l.add(-1);
			return l;
		} catch (Exception e1) {
			throw new OperationsException(
					"Can not load file.",
					e1.fillInStackTrace());
		}
	}
	private void saveObject(String filePath,Object obj) 
	throws OperationsException{
		try {	
			FileToolkit.saveFileFromObject(filePath, obj);
		} catch (IOException e) {
			throw new OperationsException(
					"Can not save file.",
					e.fillInStackTrace());
		}
	}
	public List<?> batchInsert(List<?> l) 
	throws OperationsException {
		if ( null == l || l.size()==0 ) return null;
		String objFolder = getObjectFolder(l.get(0));
		objFileIndex = loadObjectFileIndex(objFolder);
		int index = 1;
		String path = null;
		for(Object o:l) {
			if ( o instanceof Serializable ) {}
			else 
				throw new OperationsException(
						"Can not save unserializable file.");
			path = objFolder+"/"+index+EX_FILE_NAME;
			saveObject(path,o);
			index += 1;
			objFileIndex.add(index);
			this.addOrUpdateIndexItems(o.getClass(), o, path);
		}
		saveObject(objFolder+"/index"+EX_FILE_NAME,objFileIndex);		
		objFileIndex = null;
		return l;
	}
	public List<?> batchUpdate(Object obj,Predicate p,boolean replaceObject) throws OperationsException {		
		return batchUpdate(obj,p,replaceObject,0);
	}
	@SuppressWarnings("unchecked")
	private List<?> batchUpdate(Object obj,Predicate p,boolean replaceObject,int count) throws OperationsException {		
		if ( null == obj || null == p ) return null;
		Map<String,?> m = query2(p,obj.getClass());
		if ( null == m ) return null;
		List<?> i = (List<Integer>) m.get(INDEX_KEY);
		List<?> l = (List<?>) m.get(LIST_KEY);
		String of = getObjectFolder(obj);
		int size = count;
		if( count < 1 ) size = i.size();
		//If user who wants search all files 
		//can set 'count=0'.
		String path = null;
		if ( replaceObject )
			for(int j = 0 ; j < size ; j++ ) {
				path = of+"/"+i.get(j)+EX_FILE_NAME;
				saveObject(path, obj);
				this.addOrUpdateIndexItems(obj.getClass(), obj, path);
			}
		else
			for(int j = 0 ; j < size ; j++ ){
				path = of+"/"+i.get(j)+EX_FILE_NAME;
				updateObject(obj,l.get(j));
				saveObject(path, l.get(j));
				this.addOrUpdateIndexItems(obj.getClass(), obj, path);
			}		
		i = null;
		m = null;
		return l;
	}
	@SuppressWarnings("unchecked")
	public int delete(Predicate p,Class<?> clazz) throws OperationsException {
		int delCount = 0;
		if ( null == clazz || null == p ) return delCount;
		Map<String,?> m = query2(p,clazz);
		if ( null == m ) return delCount;
		List<Integer> i = (List<Integer>) m.get(INDEX_KEY);
		if ( i.size() == 0 ) return delCount;
		String of = getObjectFolder(clazz);
		objFileIndex = loadObjectFileIndex(of);
		File f = null;
		Object obj = null;
		String path = null;
		for ( Integer j : i ){
			path = of+"/"+j+EX_FILE_NAME;
			f = new File(path);
			if ( null != f ) {
				if ( f.exists() && f.isFile() ) {
					obj = this.safeLoadObject(path);
					this.removeIndexItems(clazz, obj);
					f.delete();
					objFileIndex.set(j, -1);
					delCount++;
				}
			}
		}
		saveObject(of+"/index"+EX_FILE_NAME,objFileIndex);
		objFileIndex = null;
		return delCount;
	}
	public Object insert(Object obj) throws OperationsException {
		if ( null == obj ) return null;
		String objFolder = getObjectFolder(obj);
		objFileIndex = loadObjectFileIndex(objFolder);
		if ( obj instanceof Serializable ) {}
		else 
			throw new OperationsException(
					"Can not save unserializable file.");
		int index = objFileIndex.size();
		objFileIndex.add(index);
		String path = objFolder+"/"+index+EX_FILE_NAME;
		saveObject(path,obj);
		this.addOrUpdateIndexItems(obj.getClass(), obj, path);
		saveObject(objFolder+"/index"+EX_FILE_NAME,objFileIndex);		
		objFileIndex = null;
		return obj;
	}
	public List<?> query(Predicate p,Class<?> clazz) throws OperationsException {
		String of = getObjectFolder(clazz);
		objFileIndex = loadObjectFileIndex(of);
		if ( objFileIndex.size() == 0 ) return null;
		List<Object> l = new ArrayList<Object>();
		Object obj;
		for ( Integer i : objFileIndex )
			if ( i > 0 ) {
				obj = safeLoadObject(of+"/"+i+EX_FILE_NAME);
				if ( null == obj ) i = 0;
				else {
					if ( p.match(obj) ) l.add(obj);
				}
			}			
		objFileIndex = null;
		return l.size()==0?null:l;
	}
	private <T> Map<String,?> query2(Predicate p,Class<?> clazz) throws OperationsException {
		String of = getObjectFolder(clazz);
		objFileIndex = loadObjectFileIndex(of);
		if ( objFileIndex.size() == 0 ) return null;
		Map<String,List<?>> m = new HashMap<String,List<?>>();
		List<Integer> in = new ArrayList<Integer>();
		List<Object> l = new ArrayList<Object>();
		Object obj;
		for ( Integer i : objFileIndex )
			if ( i > 0 ) {
				obj = safeLoadObject(of+"/"+i+EX_FILE_NAME);
				if ( null == obj ) i = 0;
				else {
					if ( p.match(obj) ) {
						l.add(obj);
						in.add(i);
					}
				}
			}		
		if ( in.size() == 0 || l.size() == 0 ) return null;
		m.put(INDEX_KEY, in);
		m.put(LIST_KEY, l);
		objFileIndex = null;
		return m;
	}
	
	public FileAction setDBFolder(URI uri) throws OperationsException {
		if ( null != uri ) dbFolder = uri.getPath();
		return this;
	}
	public Object update(Object obj,Predicate<?> p,boolean replaceObject) throws OperationsException {
		List<?> l = batchUpdate(obj,p,replaceObject,1);
		return l==null||l.size()==0?null:l.get(0);
	}
	
	/**
	 * Update parameters of object.
	 * @param source
	 * @param target
	 * @return success or failure
	 * */
	private boolean updateObject(Object source, Object target) throws OperationsException{
		if ( source.getClass() != target.getClass() ) return false;
		Class<?> clazz = source.getClass();
		Field[] fields = clazz.getDeclaredFields();
		Object v;
		for ( Field f : fields ) {
			int modifier = f.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				;//no code here
			} else {	
				v = ReflectToolkit.getFieldValue(f, source);
				if ( v == null ) continue;
				ReflectToolkit.setFieldValue(f.getName(), f.getType(),target,v);
			}
				
		}
		return true;
	}
	
	/**
	 * Return null when file does not exist.
	 * @param fullPath
	 * @return Object
	 * */
	private Object safeLoadObject(String fullPath) throws OperationsException{
		if ( null == fullPath || "".equals(fullPath) ) return null;
		File f = new File(fullPath);
    	if ( !f.exists() || !f.isFile() ) return null;
		
		try {
	    	FileInputStream fis = new FileInputStream(f);
	    	ObjectInputStream ois = new ObjectInputStream(fis);
	    	Object obj = ois.readObject();
	    	ois.close();
	    	fis.close();
	    	ois = null;
	    	fis = null;
	   	 	f = null;
	   	 	return obj;
		} catch (FileNotFoundException e) {
			throw new OperationsException(
					"Can not load file to object because of missing file.",
					e.fillInStackTrace());
		} catch (IOException e) {
			throw new OperationsException(
					"Can not load file to object because of io mistake.",
					e.fillInStackTrace());
		} catch (ClassNotFoundException e) {
			throw new OperationsException(
					"Can not load file to object because of wrong class.",
					e.fillInStackTrace());
		}
	}

	public SeFileAction setChildFolder(String childFolder) {
		this.childFolder = childFolder;
		return this;
	}

	public TreeMap<?, ?> createIndex(String indexName,Class<?> clazz,Field field)
	throws OperationsException {
		if ( null == indexName || null == clazz || null == field ) return null;
		TreeMap<?,?> tm = getTreeMap(clazz, indexName);
		if ( null != tm ) return tm;
		tm = new TreeMap<Object, Object>();
		boolean b = this.setTreeMap(tm, clazz, field,indexName);
		if ( !b ) throw new OperationsException("It is fail to create index store.");
		return tm;
	}
	
	public TreeMap<?, ?> getIndex(Class<?> clazz,String indexName) throws OperationsException {
		return getTreeMap(clazz, indexName);
	}
	
	public boolean removeIndex(Class<?> clazz, String indexName)
		throws OperationsException {
		Map<String,IndexStoreBean> store = getIndexStore(clazz);
		if ( null == store || null  == store.get(indexName) ) return false;
		File f = new File(store.get(indexName).getFilePath());
		if ( f.exists() && f.isFile() ) f.delete();
		store.remove(indexName);
		return true;
	}

	//---------------------------------------------------------------------------------

	private IndexStoreBean getIndexStoreBean(Class<?> clazz,String name) 
	throws OperationsException{
		Map<String,IndexStoreBean> m = getIndexStore(clazz);
		if ( null == m ) return null;
		return m.get(name);
	}
	
	@SuppressWarnings({ "unchecked", "unused" })
	private void addOrUpdateIndexItems(Class<?> clazz,Object obj,String filePath) 
	throws OperationsException{
		Map<String,IndexStoreBean> indexStore = getIndexStore(clazz);
		String[] keys = (String[]) indexStore.keySet().toArray();
		IndexStoreBean bean = null;
		TreeMap<String,Object> tm = null;
		for( String key : keys ){
			bean = indexStore.get(key);
			tm = (TreeMap<String,Object>) safeLoadObject(bean.getFilePath());
			if ( null == tm ) createIndex(key,clazz,bean.getField());
			tm.put(ReflectToolkit.getFieldValue(bean.getField(), obj).toString(),filePath);
			saveObject(bean.getFilePath(),obj);
		}
		
		
	}
	
	@SuppressWarnings({ "unchecked", "unused" })
	private void removeIndexItems(Class<?> clazz,Object obj) 
	throws OperationsException{
		Map<String,IndexStoreBean> indexStore = getIndexStore(clazz);
		String[] keys = (String[]) indexStore.keySet().toArray();
		IndexStoreBean bean = null;
		TreeMap<String,Object> tm = null;
		for( String key : keys ){
			bean = indexStore.get(key);
			tm = (TreeMap<String,Object>) safeLoadObject(bean.getFilePath());
			if ( null == tm ) createIndex(key,clazz,bean.getField());
			tm.remove(ReflectToolkit.getFieldValue(bean.getField(), obj).toString());
			saveObject(bean.getFilePath(),obj);
		}
	}
	
	@SuppressWarnings("unchecked")
	private TreeMap<String,?> getTreeMap(Class<?> clazz,String name) 
	throws OperationsException{
		IndexStoreBean bean = getIndexStoreBean(clazz,name);
		if ( null == bean ) return null;
		return (TreeMap<String,?>) safeLoadObject(bean.getFilePath());
	}
	
	private Map<String,IndexStoreBean> createIndexStore(Class<?> clazz) throws OperationsException{
		String s = getObjectFolder(clazz);
		if ( null == s || "".equals(s) ) return null;
		s += "/"+INDEX_STORE;
		Map<String,IndexStoreBean> map = new HashMap<String,IndexStoreBean>();
		try {
			FileToolkit.saveFileFromObject(s, map);
		} catch (IOException e) {
			throw new OperationsException(
					"Can not save file.",
					e.fillInStackTrace());
		}
		return map;
	}
	
	private boolean setTreeMap(TreeMap<?,?> tm,Class<?> clazz,Field field,String name) 
	throws OperationsException{
		if ( null == tm ) return false;
		Map<String,IndexStoreBean> m = getIndexStore(clazz);
		if ( null == m ) {
			m = createIndexStore(clazz);
			if ( null == m ) return false;
		}
		
		String s = getObjectFolder(clazz);
		if ( null == s || "".equals(s) ) return false;
		s += "/"+name+INDEX_EX_FILE_NAME;	
		try {
			FileToolkit.saveFileFromObject(s, tm);
		} catch (IOException e) {
			throw new OperationsException(
					"Can not save file.",
					e.fillInStackTrace());
		}
		
		IndexStoreBean bean = new IndexStoreBean();
		bean.setFilePath(s);
		bean.setClazz(clazz);
		bean.setField(field);
		bean.setName(name);
		m.put(name, bean);
		return true;
	}
	
	@SuppressWarnings("unchecked")
	private Map<String,IndexStoreBean> getIndexStore(Class<?> clazz) throws OperationsException{
		String s = getIndexStorePath(clazz);
		return (Map<String,IndexStoreBean>)safeLoadObject(s);
	}
	
	private String getIndexStorePath(Class<?> clazz) {
		String s = getObjectFolder(clazz);
		if ( null == s || "".equals(s) ) return null;
		s += "/"+INDEX_STORE;
		return s;
	}
	
	
	
}
