/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.properties;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.AbstractFilesOperation;
import com.nonesole.persistence.file.IFileTransformer;
import com.nonesole.persistence.file.TransformerFactory;
import com.nonesole.persistence.tools.DateToolkit;
import com.nonesole.persistence.tools.FileToolkit;

/**
 * Properties file operations
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-20
 */
public class PropertiesOperation extends AbstractFilesOperation {

	private static final PropertiesOperation PO = new PropertiesOperation();
	
	private static IFileTransformer ift = null;
	
	static {
		try {
			ift = TransformerFactory
				.getInstance()
				.getFileTransformer(TransformerFactory.PROPERTIES_FILE);
		} catch (OperationsException e) {
			new OperationsException(
					"Getting file transformer is failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Get Properties operation object
	 * */
	public static PropertiesOperation getInstance(){
		return PO;
	}
	
	protected String getExtendFileName(){
		return TransformerFactory.PROPERTIES_FILE_END;
	}
	
	protected void saveFile(String fullFilePath,Object obj) 
	throws OperationsException {
		OutputStream os;
		try {
			os = new BufferedOutputStream(
					new FileOutputStream(fullFilePath));
		} catch (FileNotFoundException e) {
			throw new OperationsException(
					"Can not open file before saving it.",e.fillInStackTrace());
		}
		
		Properties pro = (Properties)ift.getContent(obj);
		StringBuilder comment = new StringBuilder();
		comment.append("File maker : ")
		       .append(PropertiesOperation.class.toString())
			   .append(FileToolkit.LINE_SEPARATOR)
			   .append("#")
			   .append("Create time : ")
			   .append(DateToolkit.now());
		try {
			pro.store(os, comment.toString());
			os.flush();
		} catch (IOException e) {
			throw new OperationsException(
					"Can not save file.",e.fillInStackTrace());
		}
		try {
			os.close();
		} catch (IOException e) {
			throw new OperationsException(
					"Unable close OutputStream.",e.fillInStackTrace());
		}
		os = null;
	}
	
}
