/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.properties;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Properties;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.IFileTransformer;
import com.nonesole.persistence.tools.FileToolkit;
import com.nonesole.persistence.tools.FormatToolkit;
import com.nonesole.persistence.tools.ReflectToolkit;

/**
 * Properties object Transformer
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-25
 */
public class PropertiesTransformer implements IFileTransformer {

	private final static PropertiesTransformer TRANSFORMER;
	
	static {
		TRANSFORMER = new PropertiesTransformer();
	}
	
	private PropertiesTransformer(){}
	
	/**
	 * Get transformer object
	 * @return PropertiesTransformer
	 * */
	public static PropertiesTransformer getInstance(){
		return TRANSFORMER;
	}
	
	public Object getContent(Object obj) throws OperationsException{
		if ( null == obj )
			return null;
		Class<?> clazz = obj.getClass();
		Field[] fields = clazz.getDeclaredFields();
		Properties property = new Properties();
		Object value = null;
		for ( Field f : fields ) {
			int modifier = f.getModifiers();
			if ( Modifier.isFinal(modifier) ||
					(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
				//do not need set and get method if the field is public or static
			} else {		
				value = ReflectToolkit.getFieldValue(f, obj);
				property.put(f.getName(), FormatToolkit.transformToString(value));	
			}
		}
		return property;
	}

	public Object getObject(String fileName) throws OperationsException{
		if ( null == fileName || "".equals(fileName) ){
			throw new OperationsException("File name is null.");
		}
		
		Properties property = new Properties();
		InputStream input;
		try {
			input = FileToolkit.loadFileToInputStream(fileName);
			property.load(input);
		} catch (IOException e) {
			throw new OperationsException(
					"Can not open file.",e.fillInStackTrace());
		}
		return property;
	}
	
}
