/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.se;

import java.io.IOException;
import java.io.Serializable;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.AbstractFilesOperation;
import com.nonesole.persistence.file.IFileTransformer;
import com.nonesole.persistence.file.TransformerFactory;
import com.nonesole.persistence.tools.FileToolkit;

/**
 * Serialization Object Operation
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-26
 */
public class SerializationOperation extends AbstractFilesOperation {

	static final SerializationOperation SO = new SerializationOperation();
	
	public static SerializationOperation getInstance() {
		return SO;
	}
	
	private static IFileTransformer ift = null;
	
	static {
		try {
			ift = TransformerFactory
				.getInstance()
				.getFileTransformer(TransformerFactory.SERIALIZATION_FILE);
		} catch (OperationsException e) {
			new OperationsException(
					"Getting file transformer is failure.",e.fillInStackTrace());
		}		
	}
	
	protected String getExtendFileName(){
		return TransformerFactory.SERIALIZATION_FILE_END;
	}
	
	protected void saveFile(String fullFilePath,Object obj) 
	throws OperationsException {
		if ( obj instanceof Serializable ) {}
		else 
			throw new OperationsException(
					"Can not save file,the object is not a serializable one.");
		try {
			FileToolkit.saveFileFromObject(fullFilePath, ift.getContent(obj));
		} catch (IOException e) {
			throw new OperationsException(
					"Can not save file.",e.fillInStackTrace());
		} 
	}
	

}
