/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.se;

import java.io.IOException;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.IFileTransformer;
import com.nonesole.persistence.tools.FileToolkit;

/**
 * Serialization Object Transformer
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-27
 */
public class SerializationTransformer implements IFileTransformer {

	private final static SerializationTransformer TRANSFORMER;
	
	static {
		TRANSFORMER = new SerializationTransformer();
	}
	
	private SerializationTransformer(){}
	
	public static SerializationTransformer getInstance(){
		return TRANSFORMER;
	}
	
	public Object getContent(Object obj) throws OperationsException {
		return obj;
	}

	public Object getObject(String fileName) throws OperationsException {
		try {
			return FileToolkit.loadFileToObject(fileName);
		} catch (IOException e) {
			throw new OperationsException(
					"Can not load file.",e.fillInStackTrace());
		} catch (ClassNotFoundException e) {
			throw new OperationsException(
					"Can not read file.",e.fillInStackTrace());
		}
	}

}
