/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.file.xml;

import java.io.FileNotFoundException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.file.AbstractFilesOperation;
import com.nonesole.persistence.file.IFileTransformer;
import com.nonesole.persistence.file.TransformerFactory;
import com.nonesole.persistence.tools.FileToolkit;

/**
 * XML object operation
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-26
 */
public class XMLOperation extends AbstractFilesOperation {

	private static final XMLOperation XO = new XMLOperation();
		
	private static IFileTransformer ift = null;
		
	static {
		try {
			ift = TransformerFactory
				.getInstance()
				.getFileTransformer(TransformerFactory.XML_FILE);
		} catch (OperationsException e) {
			new OperationsException(
					"Getting file transformer is failure.",e.fillInStackTrace());
		}		
	}
	
	public static XMLOperation getInstance(){
		return XO;
	}
	
	@Override
	protected String getExtendFileName() {
		return TransformerFactory.XML_FILE_END;
	}

	@Override
	protected void saveFile(String fullFilePath, Object obj)
			throws OperationsException {
		URI uri;
		try {
			if ( !fullFilePath.startsWith("file:/") )
				fullFilePath = "file:/" + fullFilePath;
			uri = new URI(fullFilePath.replace("//", "/"));
		} catch (URISyntaxException e1) {
			throw new OperationsException(
					"Wrong File.",e1.fillInStackTrace());
		}
		try {
			FileToolkit.saveFileToXML(ift.getContent(obj), uri.toURL());
		} catch (FileNotFoundException e) {
			throw new OperationsException(
					"Can not save file.",e.fillInStackTrace());
		} catch (MalformedURLException e) {
			throw new OperationsException(
					"Can not save file.",e.fillInStackTrace());
		}		
	}

}
