/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.main;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Properties;

import com.nonesole.persistence.tools.FileHandler;

/**
 * Construction of environment of Persistence Frame 
 * @author JACK LEE
 * @version 1.0 - build in 2009-08-08
 */
public class FrameConfigConstruction {

	private static final FrameConfigConstruction fc;
	private static final Properties config;//variable of frame
	public static final String DEFAULT_LOG = "persistence_log";
	public static final String DEFAULT_DB_CONFIG = "persistence_db_config";
	public static final String DEFAULT_ORM_CONFIG = "persistence_orm_config";
	
	private static final String DEFAULT_BASE_PATH;
	private static final String DEFAULT_LOG_PATH ;
	private static final String DEFAULT_DB_CONFIG_PATH;
	private static final String DEFAULT_ORMAPPING_CONFIG_PATH;
	
	static {
		DEFAULT_BASE_PATH = (System.getProperty("user.home")
								.replaceAll("\\\\", "/")+"/PERSISTENCE/")
								.replaceAll(" ", "%20");
		DEFAULT_LOG_PATH = DEFAULT_BASE_PATH + "LOG/persistence.log";
		DEFAULT_DB_CONFIG_PATH = DEFAULT_BASE_PATH + "CONFIG/connection.xml";
		DEFAULT_ORMAPPING_CONFIG_PATH = DEFAULT_BASE_PATH + "CONFIG/persistence.xml";
		fc = new FrameConfigConstruction();
		config = FrameEnvService.getInstance().getFrameEnv();
		try {
			//initialize properties file
			if ( null == config.get(DEFAULT_LOG) ) {
				FrameEnvService.getInstance()
					.setProperty(DEFAULT_LOG, DEFAULT_LOG_PATH);
			} 
			if ( null == config.get(DEFAULT_DB_CONFIG) ) {
				FrameEnvService.getInstance()
					.setProperty(DEFAULT_DB_CONFIG, DEFAULT_DB_CONFIG_PATH);
			} 
			if ( null == config.get(DEFAULT_ORM_CONFIG) ) {
				FrameEnvService.getInstance()
					.setProperty(DEFAULT_ORM_CONFIG, DEFAULT_ORMAPPING_CONFIG_PATH);
			} 
		} catch (IOException e) {
			e.printStackTrace();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}
	
	private FrameConfigConstruction(){}
	
	/**
	 * Get FrameConfigConstruction object
	 * */
	public static FrameConfigConstruction getInstance(){
		return fc;
	}
	/**
	 * Initialize configuration
	 * */
	public void initConfig() {
		
		List<FileBean> l = MainConfigParse.getInstance().parse();
		File file = null;
		URI uri = null;
		InputStream in = null;
		for ( FileBean fb : l ){
			if ( !fb.isFile() ) {
				//folder
				file = new File(DEFAULT_BASE_PATH + fb.getName());
				log("folder : " + file.getAbsolutePath());
				if ( !file.exists() )
					file.mkdirs();
			} else {
				//file
				try {
					uri = new URI("file:/" + 
							DEFAULT_BASE_PATH + 
							fb.getAimFolder() + 
							"/" + 
							fb.getName());
				} catch (URISyntaxException e1) {
					e1.printStackTrace();
				}
				log("File : " + uri);
				if ( !fb.isCopy() ) {
					//create file
					try {
						FileHandler.getInstance().createFile(uri);
					}  catch (IOException e) {
						e.printStackTrace();
					}
				} else {
					//copy file
					in = Main.class.getResourceAsStream(fb.getSourceFolder());
					try {
						FileHandler.getInstance()
							.saveFileFromInputStream(uri, in);
					} catch (FileNotFoundException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}	
				}
			}
		}
		
	}
	
	private void log(Object obj){
		System.out.println(obj);
	}

}
