/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.main;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Properties;

import com.nonesole.persistence.tools.FileHandler;

/**
 * Provide service of initialization configuration of environment<br>
 * Notice: <br>
 * 		Value of property (key=frame.config) can not be modified.  
 * @author JACK LEE
 * @version 1.0 - build in 2009-08-07
 */
public class FrameEnvService {

	private static final FrameEnvService PS;
	private static final Properties ENV;
	private static final String CONFIG_PATH;
	private static URI uri;//URI of configuration
	private static final String FRAME_CONFIG = "frame.config";
	private static final String SPACE = "%20";
	static {
		CONFIG_PATH = (System.getProperty("user.home")
						.replaceAll("\\\\", "/")+
							"/PERSISTENCE/CONFIG/config.properties")
						.replaceAll(" ", SPACE);		
		ENV = new Properties();
		createFrameConfig();
		PS = new FrameEnvService();
	}
	
	private FrameEnvService(){}
	
	/**
	 * Get FrameEnvService object
	 * @return FrameService object
	 * */
	public static FrameEnvService getInstance(){
		return PS;
	}
		
	/**
	 * Get properties of frame
	 * @return Properties object
	 * */
	public Properties getFrameEnv(){
		if ( !CONFIG_PATH.equals(ENV.get(FRAME_CONFIG)) ) {
			createFrameConfig();
		}
		return ENV;
	}
	
	/**
	 * Get properties of key
	 * @return Properties object
	 * */
	public String getProperty(String key){
		if ( !CONFIG_PATH.equals(ENV.get(FRAME_CONFIG)) ) {
			createFrameConfig();
		}
		return ENV.getProperty(key);
	}
	
	/**
	 * Outside software can register information in frame environment 
	 * @param key 
	 * @param value 
	 * */
	public void setProperty(String key,String value) 
	throws IOException, URISyntaxException{
		if ( FRAME_CONFIG.equals(key) )
			throw new IOException(FRAME_CONFIG + " is not allowed to rewrite.");
		createFrameConfig();
		ENV.setProperty(key, value.replaceAll("%20", " "));
		ENV.store(FileHandler.getInstance().loadFileToOutputStream(uri, false), "Persistence frame config");
	}
		
	/**
	 * Create new configuration of frame environment
	 * */
	private static  void createFrameConfig() {
		try {
			uri = new URI("file:/"+CONFIG_PATH);	
			FileHandler.getInstance().createFile(uri);
			copy(FileHandler.getInstance().loadFileToProperties(uri));
			ENV.setProperty(FRAME_CONFIG, CONFIG_PATH.replaceAll("%20", " "));
		} catch (IOException e) {
			e.printStackTrace();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		} 
	}
		
	/**
	 * Copy outside properties to current environment
	 * @param source Properties object
	 * */
	private static void copy(Properties source){
		if ( null != source ) {
			Object[] keys = source.keySet().toArray();
			for ( Object key : keys ) {
				ENV.put(key, (null==source.get(key))?"":source.get(key));
			}
		}
	}
	
}
