/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.main;

import java.io.IOException;
import java.io.InputStream;

import com.nonesole.persistence.connectionpool.ConnectionPoolServer;
import com.nonesole.persistence.exception.OperationsException;

/**
 * Entrance of current jar package<br>
 * It can initialize frame environment , start
 * connection pool service and close connection pool service
 * @author JACK LEE
 * @version 1.0 - build in 2009-08-05
 */
public class Main {
	
	/**
	 * Entrance of jar
	 * <pre>
	 * Service
	 *    1initialize frame environment
	 *    2start connection pool service
	 *    3close connection pool service
	 * </pre>
	 * @param args
	 */
	public static void main(String[] args) {
		if ( null == args || args.length == 0) {
			Main.getInstance().help();
			return;
		}
		
		StringBuilder sb = new StringBuilder();
		for ( String arg : args )
			sb.append(arg).append(" ");
		sb.deleteCharAt(sb.length()-1);
		String s = sb.toString();
		
		if ( null != s ) {
			if ( "init config".equals(s) ) {
				Main.getInstance().initConfig();
			} else if ( "start server".equals(s) ) {
				Main.getInstance().startPool();
			} else if ( "stop server".equals(s) ) {
				Main.getInstance().stopPool();
			} else if ( "help".equals(s) || "?".equals(s) ) {
				Main.getInstance().help();
			}
		}
	}
	
	private static final Main main;
	
	static {
		main = new Main();
	}
		
	/**
	 * Construction
	 * */
	private Main(){}
		
	/**
	 * Return main object
	 * */
	public static Main getInstance(){
		return main;
	}
	
	/**
	 * Print help
	 * */
	private void help(){
		log("Help notes:");
		log("  1. Command 'init config' will initialize frame config files. ");
		log("  2. Command 'start pool' will start connection pool service. ");
		log("  3. Command 'stop pool' will stop connection pool service. ");
	}
	
	/**
	 * Initialize configuration files of frame
	 * */
	private void initConfig(){
		log("This operation will rewrite the target files.Do you want to continue?");
		byte[] b = null;
		InputStream is = null;
		while ( true ) {
			System.out.print("Your choice(Y|N): ");
			try {
				b = new byte[3];
				is = System.in;
				is.read(b);
			} catch (IOException e) {
				e.printStackTrace();
			}
			if ( "Y\r\n".equals(new String(b)) 
					|| "y\r\n".equals(new String(b)) ) {
				FrameConfigConstruction.getInstance().initConfig();
				break;
			} else if ( "N\r\n".equals(new String(b)) 
					|| "n\r\n".equals(new String(b)) ){
				break;
			} else {
				try {
					boolean handled = false;
					for ( byte bt : b ) {
						if ( '\n'==bt ) {
							handled = true;
							break;
						}
					}
					if ( !handled ) {
						b = new byte[1];
						do {
							b[0] = (byte) is.read();
							if ( '\n'==b[0] ) {
								break;
							} 
						} while ( b[0] != -1 );
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	/**
	 * Start service of connection pool
	 * */
	private void startPool(){
		try {
			ConnectionPoolServer.start();
		} catch (OperationsException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Close service of connection pool
	 * */
	private void stopPool(){
		ConnectionPoolServer.stop();
	}
	
	/**
	 * Print log
	 * */
	private static void log(Object obj){
		System.out.println(obj);
	}
	
}
