/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.main;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Vector;

import com.nonesole.persistence.tools.FormatToolkit;

/**
 * Parser of configs.properies file
 * @author JACK LEE
 * @version 1.0 - build in 2009-08-07
 */
public class MainConfigParse {

	private static final MainConfigParse parse;
	static {
		parse = new MainConfigParse();
	}
	private MainConfigParse(){}
	public static MainConfigParse getInstance(){
		return parse;
	}
	
	/**
	 * Parse configuration and get files and folder which need initialize.
	 * @return List<FileBean>
	 * */
	public List<FileBean> parse(){
		Properties prop = new Properties();
		InputStream in = Main.class.getResourceAsStream("configs.properties");
		try {
			prop.load(in);
		} catch (IOException e) {
			e.printStackTrace();
		}
		Integer[] sconfig = parseSystemConfig(prop);
		return parseFileConfig(prop,sconfig);
	}
	
	/**
	 * Parse content by flag of folder
	 * @param sysparam 
	 * @return Upper and lower limits of flags
	 * */
	private int[] parseFolderFlagDomain(Properties prop){
		return parseObjectFlagDomain("2",prop);
	}
	
	/**
	 * Parse content by flag of file
	 * @param sysparam 
	 * @return Upper and lower limits of flags
	 * */
	private int[] parseFileFlagDomain(Properties prop){
		return parseObjectFlagDomain("3",prop);
	}
	
	/**
	 * Parse object by flag 
	 * @param sysparam 
	 * @return Upper and lower limits of flags
	 * */
	private int[] parseObjectFlagDomain(String sysparam,Properties prop){
		Object[] objs = prop.get(sysparam).toString().split("\\|")[1].split("-");
		int[] domain = new int[2];
		domain[0]=Integer.valueOf((String)objs[0]);
		domain[1]=Integer.valueOf((String)objs[1]);
		return domain;
	}
	
	/**
	 * Parse files and folders
	 * @param prop - Properties object
	 * @param flags - Integer[] array
	 * @return list of FileBean
	 * */
	private List<FileBean> parseFileConfig(Properties prop,Integer[] flags){
		List<FileBean> l = new ArrayList<FileBean>();
		List<FileBean> fl = new ArrayList<FileBean>();
		String[] param;
		FileBean fb;
		
		int[] dic = parseFolderFlagDomain(prop);
		int[] f = parseFileFlagDomain(prop);
		
		for ( int flag : flags ){
			//Get information of operations of files and folder
			param = prop.get(String.valueOf(flag)).toString().split("\\|");
			fb = new FileBean();
			fb.setId(flag);
			int i = 0 ;
			for ( Object o : param ) {
				//format  :	key=name|action|target Folder|source file
				switch(i){
				case 0:
					fb.setName((String)o);
					break;
				case 1:
					if ( "copy".equals(prop.get(o)) )
						fb.setCopy(true);
					else
						fb.setCopy(false);
					break;
				case 2:
					fb.setAimFolder(((String)prop.get(o)).split("\\|")[0]);
					break;
				case 3:
					fb.setSourceFolder((String)prop.get(o));
					break;
				default:break;
				}
				i++;
			}
			//set if transform to file
			if ( fb.getId() >= dic[0] &&  fb.getId() <= dic[1] ){
				//file
				fb.setFile(false);
				fl.add(fb);
			} else if ( fb.getId() >= f[0] &&  fb.getId() <= f[1] )  {
				//folder
				fb.setFile(true);
				l.add(fb);
			}
		}
		//folder first,file next
		l.addAll(fl);
		return l;
	}
	
	/**
	 * Parse configuration of SystemConfig and get 
	 * flag arrays of folders and files
	 * @param prop Properties
	 * @return Integer[] array
	 * */
	private Integer[] parseSystemConfig(Properties prop){
		Object[] objs = prop.get("0").toString().split("\\|");
		Vector<Integer> v = new Vector<Integer>();
		Object[] os = null;
		for ( Object obj : objs ) {
			if ( FormatToolkit.isNumber(obj) ) {
				v.add(Integer.valueOf((String)obj));
			} else {
				os = obj.toString().split("\\-");
				for ( int i = Integer.valueOf((String)os[0]) ; 
					i <= Integer.valueOf((String)os[1]) 
						&& i >= Integer.valueOf((String)os[0]) 
					; i++ ) {
					if ( FormatToolkit.isNumber(i) ) {
						v.add(i);
					}
				}
			}
		}
		Integer[] its = new Integer[v.size()];
		int i = 0;
		for ( Integer it : v ) {
			its[i] = it;
			i++;
		}
		return its;
	}
	
}
