/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import com.nonesole.persistence.exception.OperationsException;
import com.nonesole.persistence.tools.ReflectToolkit;

/**
 * Data Query Language Class
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-15
 */
public class DQL {

	private final static DQL dql;
	static {
		dql = new DQL();
	}
	private DQL(){}
	
	/**
	 * Get DQL object
	 * @return DQL object
	 * */
	public static DQL getInstance(){
		return dql;
	}
	
	/**
	 * This method executes query sql and hides exception. 
	 * @param sql
	 * @param clazz - Class of object
	 * @param conn 
	 * */
	protected List<Object> query(String sql,Class<?> clazz,Connection conn) throws OperationsException {
		
		List<Object> l = new ArrayList<Object>();
		
		//Prevent injection attacks
		sql.replaceAll("'", "''");
		
		Statement stmt = null;
		ResultSet rs = null;
		try {
			stmt = (Statement) conn.createStatement();
			rs = stmt.executeQuery(sql);
			Object obj = clazz.newInstance();
			while( rs.next()) {
				obj = clazz.newInstance();
				//transform ResultSet to object
				ReflectToolkit.parseResultSetToObject(rs, obj);
				l.add(obj);
			}
			
			return l;
		} catch (SQLException e) {
			try {
				conn.rollback();
			} catch (SQLException e1) {
				throw new OperationsException(
						"Executing sql is failure.",e.fillInStackTrace());
			}
			throw new OperationsException(
					"Executing sql is failure.",e.fillInStackTrace());
		} catch (Exception e) {
			throw new OperationsException(
					"Executing sql is failure.",e.fillInStackTrace());
		} finally {
			if ( conn != null ) {
				try {
					rs.close();
					rs = null;
					stmt.close();
					stmt = null;
					conn.close();
					conn = null;
				} catch (SQLException e) {
					throw new OperationsException(
							"Executing sql is failure.",e.fillInStackTrace());
				}
			}
		}
		
	}
	
}
