/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.sql;

import java.sql.Connection;
import java.util.List;

import com.nonesole.persistence.exception.OperationsException;

/**
 * SQL operation interface
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-11
 */
public interface ISQLOperations  {

	/**
	 * Insert one object to database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object insert(Object obj,Connection conn) throws OperationsException;
	
	/**
	 * Update one object in database
	 * @param obj 
	 * @param conn
	 * @return object
	 * */
	public Object update(Object obj,Connection conn) throws OperationsException;
	
	/**
	 * Delete one object in database
	 * @param obj 
	 * @param conn
	 * */
	public void delete(Object obj,Connection conn) throws OperationsException;
		
	/**
	 * Search objects in database
	 * @param sql 
	 * @param clazz - Class of object
	 * @param conn
	 * @return object
	 * */
	public List<Object> query(String sql,Class<?> clazz,Connection conn) throws OperationsException;
		
	
	/**
	 * Execute sql (it would be insert,update,delete sql)
	 * @param sql
	 * @param conn
	 * */
	public void execute(String sql,Connection conn) throws OperationsException;
	

	/**
	 * Insert list of objects to database
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<Object> batchInsert(List<Object> obj,Connection conn) throws OperationsException;
	
	/**
	 * Update list of objects in database
	 * @param list of obj 
	 * @param conn
	 * @return list of object
	 * */
	public List<Object> batchUpdate(List<Object> obj,Connection conn) throws OperationsException;
	
	/**
	 * Delete list of objects in database
	 * @param list of obj 
	 * @param conn
	 * */
	public void batchDelete(List<Object> obj,Connection conn) throws OperationsException;
	
	
}
