/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import com.nonesole.persistence.exception.OperationsException;

/**
 * Deal with object with blob.
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-26
 */
public class PreparedStatementOperations implements ISQLOperations{

	private final static  PreparedStatementOperations BO;
	
	private final static PreparedStatementDML PSDML;
	
	private final static SQLOperations SQLOP;
	
	static {
		BO = new PreparedStatementOperations();
		PSDML = PreparedStatementDML.getInstance();
		SQLOP = SQLOperations.getInstance();
	}
	
	private PreparedStatementOperations(){}
	
	public static PreparedStatementOperations getInstance(){
		return BO;
	}
	
	public void batchDelete(List<Object> obj, Connection conn)
			throws OperationsException {
		SQLOP.batchDelete(obj, conn);		
	}

	public List<Object> batchInsert(List<Object> obj, Connection conn)
			throws OperationsException {
		try {
			conn.setAutoCommit(false);
			for ( Object o : obj )
				PSDML.getInsertConnection(conn, o);
			conn.commit();
		    conn.close();
		} catch (SQLException e) {
			throw new OperationsException("Batch insert is failed.");
		}
		return obj;
	}

	public List<Object> batchUpdate(List<Object> obj, Connection conn)
			throws OperationsException {
		try {
			conn.setAutoCommit(false);
			for ( Object o : obj )
				PSDML.getUpdateConnection(conn, o);
			conn.commit();
		    conn.close();
		} catch (SQLException e) {
			throw new OperationsException("Batch insert is failed.");
		}
		return obj;
	}

	public void delete(Object obj, Connection conn) throws OperationsException {
		SQLOP.delete(obj, conn);	
	}

	public void execute(String sql, Connection conn) throws OperationsException {
		SQLOP.execute(sql, conn);
	}

	public Object insert(Object obj, Connection conn)
			throws OperationsException {
		try {
			conn.setAutoCommit(false);
			PSDML.getInsertConnection(conn, obj);
			conn.commit();
		    conn.close();
		    conn = null;
		} catch (SQLException e) {
			throw new OperationsException("Insert is failed.");
		}
		return null;
	}

	public List<Object> query(String sql, Class<?> clazz, Connection conn)
			throws OperationsException {
		return SQLOP.query(sql, clazz, conn);
	}

	public Object update(Object obj, Connection conn)
			throws OperationsException {
		try {
			conn.setAutoCommit(false);
			PSDML.getUpdateConnection(conn, obj);
			conn.commit();
		    conn.close();
		    conn = null;
		} catch (SQLException e) {
			throw new OperationsException("Batch insert is failed.");
		}
		return obj;
	}
	
}
