/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.sql;

import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;

import com.nonesole.persistence.exception.OperationsException;

/**
 * Implements ISQLOperations,transform object into sql 
 * and execute it.
 * @author JACK Lee
 * @version 1.0 - build in 2009-06-29
 */
public class SQLOperations implements ISQLOperations{
		
	private final static SQLOperations operation;
	
	private SQLOperations(){}
	
	static {
		operation = new SQLOperations();
	}
	
	/**
	 * Get SQLOperations object
	 * @return Operations
	 * */
	public static SQLOperations getInstance(){
		return operation;
	}
	
	public void execute(String sql,Connection conn) throws OperationsException {
		SQLDML.getInstance().execute(sql, conn);
	}

	public void delete(Object obj,Connection conn) throws OperationsException {
		String str = SQLDML.getInstance().transferToDeleteSql(obj);
		execute(str,conn);
	}

	public Object insert(Object obj,Connection conn) throws OperationsException {
		String str = SQLDML.getInstance().transferToInsertSql(obj);
		execute(str,conn);
		return obj;
	}

	public Object update(Object obj,Connection conn) throws OperationsException {
		String str = SQLDML.getInstance().transferToUpdateSql(obj);
		execute(str,conn);
		return obj;
	}

	public List<Object> query(String sql, Class<?> clazz,Connection conn)
			throws OperationsException {
		return DQL.getInstance().query(sql, clazz, conn);
	}
		
	public void batchDelete(List<Object> obj,Connection conn) throws OperationsException {
		if ( null == obj )
			throw new OperationsException("Cannot delete objects because objects are null.");		
		List<String> list = new ArrayList<String>();//list of sql
		for ( Object o : obj )
			list.add(SQLDML.getInstance().transferToDeleteSql(o));
		SQLDML.getInstance().beathExecute(list, conn);
	}

	public List<Object> batchInsert(List<Object> obj,Connection conn)
			throws OperationsException {
		if ( null == obj )
			throw new OperationsException("Cannot insert objects because objects are null.");
		List<String> list = new ArrayList<String>();//list of sql
		for ( Object o : obj )
			list.add(SQLDML.getInstance().transferToInsertSql(o));
		SQLDML.getInstance().beathExecute(list, conn);
		return obj;
	}

	public List<Object> batchUpdate(List<Object> obj,Connection conn)
			throws OperationsException {
		if ( null == obj )
			throw new OperationsException("Cannot update objects because objects are null.");
		List<String> list = new ArrayList<String>();//list of sql
		for ( Object o : obj )
			list.add(SQLDML.getInstance().transferToUpdateSql(o));
		SQLDML.getInstance().beathExecute(list, conn);
		return obj;
	}

}
