/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.sql;

/**
 * SQL operations factory.
 * Outside can get classes which have same interface 
 * but different realization.
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-20
 */
public class SQLOperationsFactory {
	
	private static  final SQLOperationsFactory factory;
	
	static {
		factory = new SQLOperationsFactory();
	}
	
	private SQLOperationsFactory(){}
	
	public static SQLOperationsFactory getInstance(){
		return factory;
	}
	
	/**
	 * This bean will transform object to a pure string of sql 
	 * and execute it.
	 * @return ISQLOperations object
	 * */
	public ISQLOperations getSQLOperations(){
		return SQLOperations.getInstance();
	}
	
	/**
	 * This bean will transform object to a prepared sql and 
	 * execute it.It supports saving blob data,Objects which 
	 * have blob data have to save blob data as byte array.(byte[]) 
	 * @return ISQLOperations object
	 * */
	public ISQLOperations getBlobOperations(){
		return PreparedStatementOperations.getInstance();
	}
	
}
