/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.tools;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Date Tools
 * @author JACK LEE
 * @version 1.0 - build in 2008-03-26
 * */
public class DateToolkit {
	
	/**
	 * Transform time to string
	 * @param Date 
	 * @param format like "yyyy-MM-dd"+ " " +"hh:mm:ss"
	 * @return String 
	 * */
	public static String timeToString(Date date,String format){
		if( date == null )
			date = new Date(System.currentTimeMillis());
		SimpleDateFormat formatDate = new SimpleDateFormat(format); 
		String strDate = formatDate.format(date);
		return strDate;
	}
	
	/**
	 * Transform time to "yyyy-MM-dd"+ " " +"hh:mm:ss"
	 * @param Date 
	 * @return String 
	 * */
	public static String nowString(Date date){
		String format = "yyyy-MM-dd"+ " " +"HH:mm:ss";
		return timeToString(date,format);
	}
	
	/**
	 * Get current time
	 * @return String 
	 * */
	public static String now(){
		Date now = Calendar.getInstance().getTime();
		String strNow = nowString(now);
		return strNow;
	}
	
	/**
	 * Get current time include ms
	 * @return String 
	 * */
	public static String currentDetailTime(){
		Date now = Calendar.getInstance().getTime();
		String format = "yyyy-MM-dd"+ " " +"HH:mm:ss"+":"+"SSS";
		return timeToString(now,format);
	}
		
	/**
	 * Transform Date to "yyyy-MM-dd"
	 * @param Date 
	 * @return String 
	 * */
	public static String todayString(Date date){
		String format = "yyyy-MM-dd";
		return timeToString(date,format);
	}
	
	/**
	 * Get current date
	 * @return String 
	 * */
	public static String today(){
		Date now = Calendar.getInstance().getTime();
		String strNow = todayString(now);
		return strNow;
	}
	
	/**
	 * Transform string to Date Object
	 * @param String like "2008-1-1" or "2008-01-01"
	 * @return Date 
	 * */
	public static Date stringToDate(String strDate,String format) {
		Date newDate;
		SimpleDateFormat formatDate = new SimpleDateFormat(format);
		Date now = Calendar.getInstance().getTime();
		try {
			newDate = formatDate.parse(strDate);
		} catch (ParseException e) {
			return now;
		}
		return newDate;
	}
		
	/**
	 * Check some year whether or not is leap year 
	 * @param int year like 2009
	 * @return true - this year is leap year
	 * @throws Exception 
	 * */
	public static boolean isLeapYear(int year) throws Exception {
		if ( year < 0 || year > 10000 ) {
			year = Integer.valueOf(today().split("-")[0]);
		}
		if ( year % 4 == 0 )
			return true;
		else
			return false;
	}
		
}
