/*
    Persistence v1.04 is a easy-use O-R and O-F frame which suit local software.   
    Copyright (C) 2009 www.nonesole.com Jack Lee

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package com.nonesole.persistence.tools;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.sql.ResultSet;
import java.util.Map;

import com.nonesole.persistence.exception.OperationsException;

/**
 * Reflect Tools
 * @author JACK LEE
 * @version 1.0 - build in 2009-07-01
 */
public class ReflectToolkit {

	/**
	 * Transform Map to JavaBean
	 * @param map Map which has fields and values
	 * @param c Class of JavaBean
	 * @return JavaBean object
	 * */
	public final static Object parseMapToObject(Map<String,Object> map,Class<?> c) throws Exception{
		if ( null == map )
			return null;
		
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new Exception("Fields is null.");
		
		Object obj = c.newInstance();
		String name;//field name
		Object value;//field value
		Class<?> type = null;
		for( Field field : fields ){
			name = field.getName();
			value = map.get(field.getName());
			type = field.getType();
			setFieldValue(name, type, obj,value);
		}
		
		return obj;
	}
	
	/**
	 * Transform ResultSet to JavaBean
	 * @param rs ResultSet object
	 * @param c Class of JavaBean
	 * @return JavaBean object
	 * */
	public final static void parseResultSetToObject(ResultSet rs,Object obj) throws Exception{
		if ( null == rs )
			throw new Exception("ResultSet is null.");
		if ( null == obj )
			throw new Exception("Object is null.");
		
		Class<?> c = obj.getClass();
		java.lang.reflect.Field[] fields = c.getDeclaredFields();
		if ( fields == null )
			throw new Exception("Fields is null.");
		
		String name;//field name
		Object value;//field value
		Class<?> type = null;
		for( Field field : fields ){
			name = field.getName();
			type = field.getType();
			value = rs.getObject(name);
			setFieldValue(name, type, obj,value);
		}
		
	}
	
	/**
	 * Get value by get method
	 * @param field Field object
	 * @param obj owner of field
	 * @throws OperationsException
	 * */
	public static Object getFieldValue(
			Field field,Object obj) 
	throws OperationsException{
		Method m = null;
		
		int modifier = field.getModifiers();
		if ( Modifier.isFinal(modifier) ||
				(Modifier.isStatic(modifier) && Modifier.isPublic(modifier) ) ) {
			return null;
		} 
		String fieldName = field.getName();
		Class<?> fieldType = field.getType();
		Class<?> classType = obj.getClass();
		StringBuilder sb = new StringBuilder();
		if ( fieldType == boolean.class ) {
		    if ( "is".equals(fieldName.substring(0,2)) )
		    	sb.append("is").
				append(fieldName.substring(2, fieldName.length()));
		    else
				sb.append("is").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		
		} else
			sb.append("get").
				append(fieldName.substring(0,1).toUpperCase()).
				append(fieldName.substring(1, fieldName.length()));
		try {
			m = classType.getMethod(sb.toString(),new Class[]{});
		} catch (SecurityException e) {
			throw new OperationsException(
					"Getting method failure.",e.fillInStackTrace());
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			throw new OperationsException(
					"Getting method failure.",e.fillInStackTrace());
		}
		try {
			return m.invoke(obj, new Object[]{});
		} catch (IllegalArgumentException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		} catch (IllegalAccessException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		} catch (InvocationTargetException e) {
			throw new OperationsException(
					"Invoking method failure.",e.fillInStackTrace());
		}
	}
	
	/**
	 * Set field value by set method of object
	 * @param fieldName field name
	 * @param fieldType field type
	 * @param obj owner of field
	 * @param parameter value of filed
	 * @throws OperationsException
	 * */
	public static void setFieldValue(
			String fieldName,Class<?> fieldType,Object obj,Object parameter) 
	throws OperationsException{
		Class<?> clazz = obj.getClass();
		Method m = null;
		if ( null != parameter ) {
			StringBuilder sb = new StringBuilder();
			if ( fieldName.startsWith("is") ) {
				fieldName = fieldName.replace("is", "");
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			} else
				sb.append("set").
					append(fieldName.substring(0,1).toUpperCase()).
					append(fieldName.substring(1, fieldName.length()));
			try {
				m = clazz.getMethod(sb.toString(),fieldType);
			} catch (SecurityException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (NoSuchMethodException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}
			//Transform string to int etc.
			if ( String.class == parameter.getClass() && String.class != fieldType ) {
				String v = parameter.toString().trim();
				if ( boolean.class == fieldType || Boolean.class == fieldType ) {
					parameter = Boolean.parseBoolean(v);
				} else if ( int.class == fieldType || Integer.class == fieldType ) {
					parameter = Integer.valueOf(v).intValue();
				} else if ( long.class == fieldType || Long.class == fieldType ) {
					parameter = Long.valueOf(v).longValue();
				} else if ( float.class == fieldType || Float.class == fieldType ) {
					parameter = Float.valueOf(v).floatValue();
				} else if ( double.class == fieldType || Double.class == fieldType ) {
					parameter = Double.valueOf(v).doubleValue();
				} else if ( short.class == fieldType || Short.class == fieldType ) {
					parameter = Short.valueOf(v).shortValue();
				} else if ( Byte.class == fieldType || byte.class == fieldType ) {
					parameter = Byte.valueOf(v).byteValue();
				} 
			}
			try {
				m.invoke(obj, new Object[]{parameter});
			} catch (IllegalArgumentException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (IllegalAccessException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			} catch (InvocationTargetException e) {
				throw new OperationsException(
						"Invoking method failure.",e.fillInStackTrace());
			}			
		}	
	}
	
}
